import java.util.UUID;
import org.vertx.java.core.AsyncResult;
import org.vertx.java.core.AsyncResultHandler;
import org.vertx.java.core.Handler;
import org.vertx.java.core.SimpleHandler;
import org.vertx.java.core.file.AsyncFile;
import org.vertx.java.core.http.HttpClient;
import org.vertx.java.core.http.HttpClientRequest;
import org.vertx.java.core.http.HttpClientResponse;
import org.vertx.java.core.streams.Pump;
import org.vertx.java.deploy.Verticle;

/**
 * Verticle used for downloading one file. This is used for a test.
 * @author mcalderoni
 */
public class DClient extends Verticle {

   @Override
   public void start() throws Exception {
      int portConnect = Config.PROXY_PORT;
      System.out.println("Started Client connecting to port " + portConnect);
      HttpClient client = vertx.createHttpClient().setPort(portConnect).setHost("localhost");
      final HttpClientRequest req = client.
              get("/bucket1/file"+Math.random(), new Handler<HttpClientResponse>() {
         @Override
         public void handle(final HttpClientResponse response) {
            System.out.println("Got response with code: " + response.statusCode);
            response.pause();
            final String filename = Config.TARGET_FILE_REPO + UUID.randomUUID().toString() + ".download";
            vertx.fileSystem().open(filename, new AsyncResultHandler<AsyncFile>() {
               @Override
               public void handle(AsyncResult<AsyncFile> ar) {
                  final AsyncFile asyncFile = ar.result;
                  final Pump pump = Pump.createPump(response, asyncFile.getWriteStream());
                  final long start = System.currentTimeMillis();
                  response.endHandler(new SimpleHandler() {
                     @Override
                     public void handle() {
                        asyncFile.close(new AsyncResultHandler<Void>() {
                           @Override
                           public void handle(AsyncResult<Void> ar) {
                              if (ar.exception == null) {
                                 long end = System.currentTimeMillis();
                                 int bytesPumped = pump.getBytesPumped();
                                 System.out.println("\n\t>>Downloaded " + bytesPumped
                                         + " bytes\n\t>>to " + filename + "\n\t>>in " + (end - start) + " ms");
                                 if (Config.FILE_SIZE != bytesPumped) {
                                    System.err.println("ERROR - LOST DATA --------------------------------");
                                 }
                              } else {
                                 ar.exception.printStackTrace(System.err);
                              }
                              System.out.println("closed");
                           }
                        });
                     }
                  });
                  pump.start();
                  response.resume();
               }
            });
         }
      });
      req.end();
   }
}
