import org.vertx.java.core.AsyncResult;
import org.vertx.java.core.AsyncResultHandler;
import org.vertx.java.core.Handler;
import org.vertx.java.core.SimpleHandler;
import org.vertx.java.core.file.AsyncFile;
import org.vertx.java.core.http.HttpClient;
import org.vertx.java.core.http.HttpClientRequest;
import org.vertx.java.core.http.HttpClientResponse;
import org.vertx.java.core.streams.Pump;
import org.vertx.java.deploy.Verticle;

/**
 * Verticle used for uploading one file. This is used for a test.
 * @author mcalderoni
 */
public class UClient extends Verticle {

   @Override
   public void start() throws Exception {
      int portConnect = Config.PROXY_PORT;
      System.out.println("Started Client connecting to port " + portConnect);
      HttpClient client = vertx.createHttpClient().setPort(portConnect).setHost("localhost");
      final HttpClientRequest req = client.
              put("/bucket1/file6" + Math.random(), new Handler<HttpClientResponse>() {
         @Override
         public void handle(HttpClientResponse response) {
            System.out.println("Got response with code: " + response.statusCode);
         }
      });
      req.setChunked(true);

      vertx.fileSystem().open(Config.FILE, new AsyncResultHandler<AsyncFile>() {
         @Override
         public void handle(AsyncResult<AsyncFile> ar) {
            final AsyncFile asyncFile = ar.result;
            Pump pump = Pump.createPump(asyncFile.getReadStream(), req);
            pump.start();

            asyncFile.getReadStream().endHandler(new SimpleHandler() {
               @Override
               public void handle() {
                  asyncFile.close(new AsyncResultHandler<Void>() {
                     @Override
                     public void handle(AsyncResult<Void> ar) {
                        if (ar.exception == null) {
                           req.end();
                           System.out.println("Closing file stream");
                        } else {
                           ar.exception.printStackTrace(System.err);
                        }
                     }
                  });
               }
            });
         }
      });
   }
}
